#version 420

uniform sampler2D DiffuseSampler;
uniform sampler2D DepthSampler;
uniform vec2 OutSize;

in vec2 texCoord;

out vec4 fragColor;

#ifndef CONSTANTS_GLSL
#define CONSTANTS_GLSL

// Application specific
const ivec2 GRID_SIZE = ivec2(1024, 705);
const int AREA_SIDE_LENGTH = int(pow(float(GRID_SIZE.x * GRID_SIZE.y / 2), 1.0 / 3.0));
const int STORAGE_SIZE = 839;
const uint MAX_COUNTER = 255u;

// Universal
const float EPSILON = 0.001;
const float PI = 3.141592654;

// Temporal

const float MAX_TEMPORAL_BLENDING = 0.05;
const uint SDF_GENERATION_FRAMES = 6u;
const float MAX_SDF_DIST = 255.0;

#endif // CONSTANTS_GLSL
#ifndef VOXELIZATION
#define VOXELIZATION

#ifndef CONSTANTS_GLSL
#define CONSTANTS_GLSL

// Application specific
const ivec2 GRID_SIZE = ivec2(1024, 705);
const int AREA_SIDE_LENGTH = int(pow(float(GRID_SIZE.x * GRID_SIZE.y / 2), 1.0 / 3.0));
const int STORAGE_SIZE = 839;
const uint MAX_COUNTER = 255u;

// Universal
const float EPSILON = 0.001;
const float PI = 3.141592654;

// Temporal

const float MAX_TEMPORAL_BLENDING = 0.05;
const uint SDF_GENERATION_FRAMES = 6u;
const float MAX_SDF_DIST = 255.0;

#endif // CONSTANTS_GLSL

ivec2 positionToCellScreen(vec3 position, out bool inside) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    ivec3 iPosition = ivec3(floor(position));
    iPosition += sides / 2;

    inside = true;
    if (clamp(iPosition, ivec3(0), sides - 1) != iPosition) {
        inside = false;
        return ivec2(-1);
    }

    int index = (iPosition.y * sides.z + iPosition.z) * sides.x + iPosition.x;
    
    int halfWidth = GRID_SIZE.x / 2;
    ivec2 result = ivec2(
        (index % halfWidth) * 2,
        index / halfWidth + 1
    );
    result.x += result.y % 2;

    return result;
}

ivec2 cellToPixelScreen(ivec2 cell, ivec2 screenSize) {
    return ivec2(round(vec2(cell) / GRID_SIZE * screenSize));
}

ivec2 positionToPixel(vec3 position, out bool inside) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    ivec3 iPosition = ivec3(floor(position));
    iPosition += sides / 2;

    inside = true;
    if (clamp(iPosition, ivec3(0), sides - 1) != iPosition) {
        inside = false;
        return ivec2(-1);
    }

    int index = (iPosition.y * sides.z + iPosition.z) * sides.x + iPosition.x;
    
    return ivec2(
        index % STORAGE_SIZE,
        index / STORAGE_SIZE
    );
}

ivec3 pixelToPosition(ivec2 pixel) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    int index = pixel.x + pixel.y * STORAGE_SIZE;
    return ivec3(
        index % sides.x,
        index / sides.x / sides.z,
        (index / sides.x) % sides.z
    ) - sides / 2;
}

#endif // VOXELIZATION

void main() {
    int discardModulo = 1;
    
    ivec2 fragCoord = ivec2(gl_FragCoord.xy);
    ivec2 cell = ivec2(texCoord * GRID_SIZE + 0.001);
    ivec2 pixel = cellToPixelScreen(cell, ivec2(OutSize));

    if (fragCoord == pixel) {

        float centerDepth = texelFetch(DepthSampler, fragCoord, 0).r;
        if (centerDepth > 0.0) {
            gl_FragDepth = centerDepth;
            fragColor = texelFetch(DiffuseSampler, fragCoord, 0);
            return;
        }

        vec4 depths = vec4(
            texelFetch(DepthSampler, fragCoord + ivec2(0, 1), 0).r,
            texelFetch(DepthSampler, fragCoord + ivec2(0, -1), 0).r,
            texelFetch(DepthSampler, fragCoord + ivec2(1, 0), 0).r,
            texelFetch(DepthSampler, fragCoord + ivec2(-1, 0), 0).r
        );
        float avgDepth = dot(depths, vec4(1)) / 4.0;
        vec4 weights = 1.0 - pow(depths - avgDepth, vec4(2.0));
        weights /= dot(weights, vec4(1));

        gl_FragDepth = dot(weights, depths);

        vec4 top    = texelFetch(DiffuseSampler, fragCoord + ivec2(0, 1), 0);
        vec4 bottom = texelFetch(DiffuseSampler, fragCoord + ivec2(0, -1), 0);
        vec4 right  = texelFetch(DiffuseSampler, fragCoord + ivec2(1, 0), 0);
        vec4 left   = texelFetch(DiffuseSampler, fragCoord + ivec2(-1, 0), 0);
        fragColor = weights.x * top + weights.y * bottom + weights.z * right + weights.w * left;
    } else {
        fragColor = texture(DiffuseSampler, texCoord);
        gl_FragDepth = texture(DepthSampler, texCoord).r;
    }
}