#version 420

#ifndef VOXELIZATION
#define VOXELIZATION

#ifndef CONSTANTS_GLSL
#define CONSTANTS_GLSL

// Application specific
const ivec2 GRID_SIZE = ivec2(1024, 705);
const int AREA_SIDE_LENGTH = int(pow(float(GRID_SIZE.x * GRID_SIZE.y / 2), 1.0 / 3.0));
const int STORAGE_SIZE = 839;
const uint MAX_COUNTER = 255u;

// Universal
const float EPSILON = 0.001;
const float PI = 3.141592654;

// Temporal

const float MAX_TEMPORAL_BLENDING = 0.05;
const uint SDF_GENERATION_FRAMES = 6u;
const float MAX_SDF_DIST = 255.0;

#endif // CONSTANTS_GLSL

ivec2 positionToCellScreen(vec3 position, out bool inside) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    ivec3 iPosition = ivec3(floor(position));
    iPosition += sides / 2;

    inside = true;
    if (clamp(iPosition, ivec3(0), sides - 1) != iPosition) {
        inside = false;
        return ivec2(-1);
    }

    int index = (iPosition.y * sides.z + iPosition.z) * sides.x + iPosition.x;
    
    int halfWidth = GRID_SIZE.x / 2;
    ivec2 result = ivec2(
        (index % halfWidth) * 2,
        index / halfWidth + 1
    );
    result.x += result.y % 2;

    return result;
}

ivec2 cellToPixelScreen(ivec2 cell, ivec2 screenSize) {
    return ivec2(round(vec2(cell) / GRID_SIZE * screenSize));
}

ivec2 positionToPixel(vec3 position, out bool inside) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    ivec3 iPosition = ivec3(floor(position));
    iPosition += sides / 2;

    inside = true;
    if (clamp(iPosition, ivec3(0), sides - 1) != iPosition) {
        inside = false;
        return ivec2(-1);
    }

    int index = (iPosition.y * sides.z + iPosition.z) * sides.x + iPosition.x;
    
    return ivec2(
        index % STORAGE_SIZE,
        index / STORAGE_SIZE
    );
}

ivec3 pixelToPosition(ivec2 pixel) {
    ivec3 sides = ivec3(AREA_SIDE_LENGTH);

    int index = pixel.x + pixel.y * STORAGE_SIZE;
    return ivec3(
        index % sides.x,
        index / sides.x / sides.z,
        (index / sides.x) % sides.z
    ) - sides / 2;
}

#endif // VOXELIZATION
#ifndef CONSTANTS_GLSL
#define CONSTANTS_GLSL

// Application specific
const ivec2 GRID_SIZE = ivec2(1024, 705);
const int AREA_SIDE_LENGTH = int(pow(float(GRID_SIZE.x * GRID_SIZE.y / 2), 1.0 / 3.0));
const int STORAGE_SIZE = 839;
const uint MAX_COUNTER = 255u;

// Universal
const float EPSILON = 0.001;
const float PI = 3.141592654;

// Temporal

const float MAX_TEMPORAL_BLENDING = 0.05;
const uint SDF_GENERATION_FRAMES = 6u;
const float MAX_SDF_DIST = 255.0;

#endif // CONSTANTS_GLSL

#line 6 98723

uniform sampler2D PrevSDF;
uniform sampler2D CurrSDF;
uniform vec2 InSize;
uniform vec2 OutSize;

in vec2 texCoord;
flat in int regenerate;
flat in ivec3 offset;

out vec4 fragColor;

void main() {
    ivec2 pixel = ivec2(gl_FragCoord.xy);
    ivec3 position = pixelToPosition(pixel);

    if (regenerate == 0) {
        position += offset;
    }

    bool inside;
    ivec2 oldPixel = positionToPixel(vec3(position), inside);
    if (!inside) {
        gl_FragDepth = 1.0;
        return;
    }

    if (regenerate == 0) {
        gl_FragDepth = texelFetch(PrevSDF, oldPixel, 0).r;
    } else {
        gl_FragDepth = texelFetch(CurrSDF, oldPixel, 0).r;
    }
}
